/**
 * DevExtreme (esm/renovation/ui/editors/check_box/utils.js)
 * Version: 21.2.0 (build 21243-0310)
 * Build date: Tue Aug 31 2021
 *
 * Copyright (c) 2012 - 2021 Developer Express Inc. ALL RIGHTS RESERVED
 * Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
 */
import {
    isMaterial,
    isCompact,
    current
} from "../../../../ui/themes";

function getThemeType() {
    var theme = current();
    return {
        isMaterialTheme: isMaterial(theme),
        isCompactTheme: isCompact(theme)
    }
}

function getDefaultIconSize() {
    var {
        isCompactTheme: isCompactTheme,
        isMaterialTheme: isMaterialTheme
    } = getThemeType();
    var defaultIconSize = 22;
    if (isCompactTheme) {
        defaultIconSize = 16
    } else if (isMaterialTheme) {
        defaultIconSize = 18
    }
    return defaultIconSize
}

function getDefaultFontSize() {
    var {
        isCompactTheme: isCompactTheme
    } = getThemeType();
    var defaultFontSize = isCompactTheme ? 12 : 16;
    return defaultFontSize
}

function getFontSizeByIconSize(iconSize) {
    var defaultFontSize = getDefaultFontSize();
    var defaultIconSize = getDefaultIconSize();
    var fontToIconSizeRatio = defaultFontSize / defaultIconSize;
    return Math.ceil(fontToIconSizeRatio * iconSize)
}
export {
    getDefaultIconSize,
    getFontSizeByIconSize
};
