/**
 * DevExtreme (renovation/ui/editors/check_box/check_box_icon.js)
 * Version: 21.2.0 (build 21243-0310)
 * Build date: Tue Aug 31 2021
 *
 * Copyright (c) 2012 - 2021 Developer Express Inc. ALL RIGHTS RESERVED
 * Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
 */
"use strict";
exports.CheckBoxIcon = exports.CheckBoxIconProps = exports.viewFunction = void 0;
var _inferno = require("inferno");
var _vdom = require("@devextreme/vdom");
var _get_computed_style = _interopRequireDefault(require("../../../utils/get_computed_style"));
var _window = require("../../../../core/utils/window");
var _style = require("../../../../core/utils/style");
var _type = require("../../../../core/utils/type");
var _utils = require("./utils");
var _excluded = ["size"];

function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
        default: obj
    }
}

function _objectWithoutProperties(source, excluded) {
    if (null == source) {
        return {}
    }
    var target = _objectWithoutPropertiesLoose(source, excluded);
    var key, i;
    if (Object.getOwnPropertySymbols) {
        var sourceSymbolKeys = Object.getOwnPropertySymbols(source);
        for (i = 0; i < sourceSymbolKeys.length; i++) {
            key = sourceSymbolKeys[i];
            if (excluded.indexOf(key) >= 0) {
                continue
            }
            if (!Object.prototype.propertyIsEnumerable.call(source, key)) {
                continue
            }
            target[key] = source[key]
        }
    }
    return target
}

function _objectWithoutPropertiesLoose(source, excluded) {
    if (null == source) {
        return {}
    }
    var target = {};
    var sourceKeys = Object.keys(source);
    var key, i;
    for (i = 0; i < sourceKeys.length; i++) {
        key = sourceKeys[i];
        if (excluded.indexOf(key) >= 0) {
            continue
        }
        target[key] = source[key]
    }
    return target
}

function _extends() {
    _extends = Object.assign || function(target) {
        for (var i = 1; i < arguments.length; i++) {
            var source = arguments[i];
            for (var key in source) {
                if (Object.prototype.hasOwnProperty.call(source, key)) {
                    target[key] = source[key]
                }
            }
        }
        return target
    };
    return _extends.apply(this, arguments)
}

function _defineProperties(target, props) {
    for (var i = 0; i < props.length; i++) {
        var descriptor = props[i];
        descriptor.enumerable = descriptor.enumerable || false;
        descriptor.configurable = true;
        if ("value" in descriptor) {
            descriptor.writable = true
        }
        Object.defineProperty(target, descriptor.key, descriptor)
    }
}

function _createClass(Constructor, protoProps, staticProps) {
    if (protoProps) {
        _defineProperties(Constructor.prototype, protoProps)
    }
    if (staticProps) {
        _defineProperties(Constructor, staticProps)
    }
    return Constructor
}

function _assertThisInitialized(self) {
    if (void 0 === self) {
        throw new ReferenceError("this hasn't been initialised - super() hasn't been called")
    }
    return self
}

function _inheritsLoose(subClass, superClass) {
    subClass.prototype = Object.create(superClass.prototype);
    subClass.prototype.constructor = subClass;
    _setPrototypeOf(subClass, superClass)
}

function _setPrototypeOf(o, p) {
    _setPrototypeOf = Object.setPrototypeOf || function(o, p) {
        o.__proto__ = p;
        return o
    };
    return _setPrototypeOf(o, p)
}
var viewFunction = function(viewModel) {
    var cssStyles = viewModel.cssStyles,
        elementRef = viewModel.elementRef;
    return (0, _inferno.createVNode)(1, "span", "dx-checkbox-icon", null, 1, {
        style: (0, _vdom.normalizeStyles)(cssStyles)
    }, null, elementRef)
};
exports.viewFunction = viewFunction;
var CheckBoxIconProps = {};
exports.CheckBoxIconProps = CheckBoxIconProps;
var CheckBoxIcon = function(_InfernoComponent) {
    _inheritsLoose(CheckBoxIcon, _InfernoComponent);

    function CheckBoxIcon(props) {
        var _this;
        _this = _InfernoComponent.call(this, props) || this;
        _this.state = {};
        _this.elementRef = (0, _inferno.createRef)();
        _this.__getterCache = {};
        _this.updateFontSize = _this.updateFontSize.bind(_assertThisInitialized(_this));
        _this.setIconFontSize = _this.setIconFontSize.bind(_assertThisInitialized(_this));
        _this.getComputedIconSize = _this.getComputedIconSize.bind(_assertThisInitialized(_this));
        return _this
    }
    var _proto = CheckBoxIcon.prototype;
    _proto.createEffects = function() {
        return [new _vdom.InfernoEffect(this.updateFontSize, [this.props.size])]
    };
    _proto.updateEffects = function() {
        var _this$_effects$;
        null === (_this$_effects$ = this._effects[0]) || void 0 === _this$_effects$ ? void 0 : _this$_effects$.update([this.props.size])
    };
    _proto.updateFontSize = function() {
        var size = this.props.size;
        if ((0, _window.hasWindow)()) {
            var newIconSize = (0, _type.isNumeric)(size) ? size : this.getComputedIconSize();
            if (newIconSize) {
                var newFontSize = (0, _utils.getFontSizeByIconSize)(newIconSize);
                this.setIconFontSize(newFontSize)
            }
        }
    };
    _proto.setIconFontSize = function(fontSize) {
        var element = this.elementRef.current;
        element.style.fontSize = "".concat(fontSize, "px")
    };
    _proto.getComputedIconSize = function() {
        var element = this.elementRef.current;
        var iconComputedStyle = (0, _get_computed_style.default)(element);
        var computedIconSize = parseInt(null === iconComputedStyle || void 0 === iconComputedStyle ? void 0 : iconComputedStyle.width, 10);
        return computedIconSize
    };
    _proto.componentWillUpdate = function(nextProps, nextState, context) {
        _InfernoComponent.prototype.componentWillUpdate.call(this);
        if (this.props.size !== nextProps.size) {
            this.__getterCache.cssStyles = void 0
        }
    };
    _proto.render = function() {
        var props = this.props;
        return viewFunction({
            props: _extends({}, props),
            elementRef: this.elementRef,
            setIconFontSize: this.setIconFontSize,
            getComputedIconSize: this.getComputedIconSize,
            cssStyles: this.cssStyles,
            restAttributes: this.restAttributes
        })
    };
    _createClass(CheckBoxIcon, [{
        key: "cssStyles",
        get: function() {
            var _this2 = this;
            if (void 0 !== this.__getterCache.cssStyles) {
                return this.__getterCache.cssStyles
            }
            return this.__getterCache.cssStyles = (size = _this2.props.size, width = (0, _style.normalizeStyleProp)("width", size), height = (0, _style.normalizeStyleProp)("height", size), {
                height: height,
                width: width
            });
            var size, width, height
        }
    }, {
        key: "restAttributes",
        get: function() {
            var _this$props = this.props,
                restProps = (_this$props.size, _objectWithoutProperties(_this$props, _excluded));
            return restProps
        }
    }]);
    return CheckBoxIcon
}(_vdom.InfernoComponent);
exports.CheckBoxIcon = CheckBoxIcon;
CheckBoxIcon.defaultProps = CheckBoxIconProps;
